/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*--------------------------------------------------------------------
                         -- C N R S --
         Laboratoire d'automatique et d'analyse des systemes
            Groupe Robotique et Intelligenxce Artificielle
                   7 Avenue du colonel Roche
                     31 077 Toulouse Cedex

  Fichier              : cFunctions.c
  Fonction             : Extra Functions for Trajectory Planning 
  Date de creation     : Aout 2005      
  Date de modification : Aout 2005
  Nb de lignes         : 835

  Auteur               : Ignacio HERRERA AGUILAR

----------------------------------------------------------------------*/

#include <stdio.h>
#include <math.h>
#include "matrixStruct.h"
#include "matrix.c"
#include "tpStruct.h"
#include "gb.h"
#include "cFunctions.h"


/* Discrete Convolution 
   convolution(dmat VR_A, dmat VR_B, dmat VR_C)
   input :      VR_A     Vector Row A 
                VR_B     Vector Row B
   output:	 VR_C     Vector Row C - convolution
   return:      TP_OK
                TP_ERROR
*/

TP_STATUS convolution(dmat VR_A, dmat VR_B, dmat VR_C )
{
  register int la, lb, lc;
  register int i,j,k;
  double aCo, bCo;
  
  /* Find length vectors */
  la = VR_A.ub2 - VR_A.lb2 + 1;
  lb = VR_B.ub2 - VR_B.lb2 + 1 ;
  lc = VR_C.ub2 - VR_C.lb2 + 1;
  
  /* Check length condition  */
  if (lc != la + lb -1) {
    fprintf(stderr, "convolution - TP_ERROR Dimension Vector Solution \n");
    return TP_ERROR;
  }
  
  if (matReset(VR_C)!=TP_OK) {
    fprintf(stderr, "convolution - TP_ERROR Reset Vector Solution \n");
    return TP_ERROR;
  }
  
  for (i=0;i<lc;i++) {
    k=0;
    for (j=i;j>=0;j--) {
	 if (j<=la) aCo = VR_A.el[0][j];
	 else aCo = 0.0;
	 
	 if (k<=lb) bCo = VR_B.el[0][k];
	 else bCo = 0.0;
	 
	 VR_C.el[0][i] = aCo*bCo + VR_C.el[0][i];
	 k++;
    }
  }
  return TP_OK;
}

/* Discrete deConvolution 
   deConvolution(dmat VR_A, dmat VR_B, dmat VR_D, dmat VR_R)
   input :      VR_A     Vector Row A 
                VR_B     Vector Row B
   output:	 VR_D     Vector Row D - deConvolution - Quotient
                VR_R     Vector Row R - remind
   return:      TP_OK
                TP_ERROR
*/

TP_STATUS deConvolution(dmat VR_A, dmat VR_B, dmat VR_D, dmat VR_R )
{
  register int la, lb, ld, lr;
  register int i,k;
  double CofA;
  
  /* Find length vectors */
  la = VR_A.ub2 - VR_A.lb2 + 1;
  lb = VR_B.ub2 - VR_B.lb2 + 1;
  ld = VR_D.ub2 - VR_D.lb2 + 1;
  lr = VR_R.ub2 - VR_R.lb2 + 1;
  
  /* Check length condition ld */
  if (ld != la - lb +1) {
    fprintf(stderr, "deConvolution - TP_ERROR Dimension Vector Solution \n");
    return TP_ERROR;
  }

  /* Check length condition lr */
  if (lr != lb -1) {
    fprintf(stderr, "deConvolution - TP_ERROR Dimension Vector Remind \n");
    return TP_ERROR;
  } 
 
  if (matReset(VR_D)!=TP_OK) {
    fprintf(stderr, "deConvolution - TP_ERROR Reset Vector Solution \n");
    return TP_ERROR;
  }

  if (matReset(VR_R)!=TP_OK) {
    fprintf(stderr, "deConvolution - TP_ERROR Reset Vector Remind \n");
    return TP_ERROR;
  }
  
  for (i=0;i<ld;i++) {
    if (i==0)
	 for (k=0;k<=lr;k++)
	   VR_R.el[0][k] = VR_A.el[0][k];

    if (VR_B.el[0][0]==0) {
	 fprintf(stderr,"deConvolution - TP_ERROR Component VR_B(1) = 0.0 \n");
	 return TP_ERROR;
    }
    VR_D.el[0][i]=VR_R.el[0][0]/VR_B.el[0][0];
    for (k=0;k<lr;k++) {
	 
	 if (k<lr-1) CofA = VR_R.el[0][k+1];
	 else      CofA = VR_A.el[0][i+lr];
	 
	 VR_R.el[0][k] = CofA - VR_D.el[0][i] * VR_B.el[0][k+1];

    }
  }
  return TP_OK;
}

/* Syntetic Division Function 
   SynteticDivision (Poly, root, Quot, &remind)
   input :      Poly     Coeff Polynomium 
                root     Root Value
   output:	 Quot     Quotient polynomium
                remind   Reminder
   return:      TP_OK
                TP_ERROR
*/

TP_STATUS SynteticDivision(dmat VR_Pol, double root, dmat VR_Quot, double *remind )
{
  int size,Nsize;
  int flag=1;
  register int i,zi=0;
  double NormFactor; 
  dmat VR_PolNorm;
  dmat VR_AuxPol;
  int errno;
  
  /* Polynomium ordre = vector's size - 1*/
  size = VR_Pol.ub2 - VR_Pol.lb2;

  /* Gets Normalization Factor and erase zeros elements of max_order vector */
  for (i=0;i<=size;i++) {
    if (VR_Pol.el[0][i]!=0 && flag==1 ) {
	 NormFactor = VR_Pol.el[0][i];
	 Nsize = size - i;
	 flag = 0;
	 zi   = i;
    }
    if (i==size && flag==1) NormFactor=1;
  }

  /* Auxiliar vector init*/
  VR_AuxPol = newdmat(0,0,0,Nsize,&errno);
  if (errno) {
    fprintf(stderr,"SynteticDivision - TP_ERROR newdmat Vr_AuxPol \n");
    return TP_ERROR;
  }

  /* Auxiliar vector without zeros */
  if (copyReducedMat(VR_Pol,0,zi,VR_AuxPol)!=TP_OK) {
    fprintf(stderr,"SynteticDivision - TP_ERROR copyReducedMat \n");
    return TP_ERROR;
  }

  /* Normalized vector Init */
  VR_PolNorm = newdmat(0,0,0,Nsize,&errno);
  if (errno) {
    fprintf(stderr,"SynteticDivision - TP_ERROR newdmat VR_PolNorm\n");
    return TP_ERROR;
  }

  /* Calcul of Normalized Vector */
  NormFactor=1/NormFactor;
  mulEscMat(NormFactor,VR_AuxPol,VR_PolNorm);

  /* Gets Quotient */
  VR_Quot.el[0][0]=VR_PolNorm.el[0][0];
  for (i=1;i<=Nsize-1;i++)
    VR_Quot.el[0][i]=VR_PolNorm.el[0][i]+ VR_Quot.el[0][i-1]*root;

  /* Gets reminder */
  *remind = VR_PolNorm.el[0][i] + VR_Quot.el[0][i-1]*root;

  freemat(VR_PolNorm);
  freemat(VR_AuxPol);

  return TP_OK;
}
 
/* CubicRSolution */
/* This function returns the REAL roots of      a X^3 + b X^2 + c X + d = 0    */
/* Note:                                                                       */
/*      IT'S NO A GENERAL SOLUTION                                             */
/*      OUR APPLICATION ONLY DEMANDS POSITIVE ROOTS                            */
/*      THEN COMPLEX ROOTS EQUALS TO -1                                        */
/*      CONFUSION PROBLEM WHEN REAL ROOTS EQUALS TO -1                         */
/*      RETURNS TP_ERROR                                                          */
/*      POSSIBLE SOLUTION, EVIDENT                                             */
/*      WHEN POL IS QUADRATIC RETURNS -1 FOR THE FIRST ROOT AND TP_ERROR          */
/*               IS MONOMIAL  RETURNS THE ROOT AND -1 FOR THE OTHERS AND TP_ERROR */
/*               IS CONSTANTS RETURNS -1 FOR THE ROOTS AND TP_ERROR               */
/*                                                                             */
/* Input :     dmat VR_POL     Coefficients Poly                               */
/* Output:     dmat VR_SOL     Roots Vector                                    */
                                
TP_STATUS cubicRSolution(dmat VR_Pol, dmat VR_Sol, TP_SELECT msgError )
{
  
  double a, b, c, d;
  double aM, bM;
  double b2_4ac, SR;
  double u,t,s, uAux;
  double X,Y;
  dmat VR_Quad;
  dmat VR_Y2;
  dmat VR_Y3;
  dmat VR_D;
  dmat VR_R;
  double modR;
  int errno;
  double reminder;
  TP_COMPLEX uComp,tComp, sComp, YComp;

  /* Verifie dimensions vector */
  if ((VR_Pol.ub2-VR_Pol.lb2) != 3 || (VR_Pol.ub1-VR_Pol.lb1) != 0) {
    fprintf(stderr,"cubicRSolution - TP_ERROR Poly Coefficients Error\n");
    return TP_ERROR;
  }

  if ((VR_Sol.ub2-VR_Sol.lb2) != 2 || (VR_Sol.ub1-VR_Sol.lb1) != 0) {
    fprintf(stderr,"cubicRSolution - TP_ERROR Dimension Solutions Error\n");
    return TP_ERROR;
  }

  /* Coefficients assignation */
  /*   a X^3 + b X^2 + c X + d = 0   */
  a = VR_Pol.el[0][0];
  b = VR_Pol.el[0][1];
  c = VR_Pol.el[0][2];
  d = VR_Pol.el[0][3];
  
  if (a==0) {
    if (b==0) {
	 if (c!=0) {
	   if (!msgError)
		fprintf(stderr,"cubicRSolution - TP_ERROR - Monomial     (X1 -1 -1)    \n");
	   VR_Sol.el[0][0]= -d/c;
	   VR_Sol.el[0][1]= -1.0;
	   VR_Sol.el[0][2]= -1.0;
	   return TP_OK;
	   }
	 else {
	   if (!msgError)
		fprintf(stderr,"cubicRSolution - TP_ERROR - Constant  (-1 -1 -1)    \n");
	   VR_Sol.el[0][0]= -1;
	   VR_Sol.el[0][1]= -1;
	   VR_Sol.el[0][2]= -1;
	   return TP_OK;
	 } /* FROM (c==0) */
    }
    else {
	 /*    b X^2 + c X + d = 0   */
	 b2_4ac = c*c - 4*b*d;
	 if (b2_4ac <0) {
	   if (!msgError)
		fprintf(stderr,"cubicRSolution - TP_ERROR - Quadratic Complex (-1 -1 -1) \n");
	   VR_Sol.el[0][0]=-1.0;
	   VR_Sol.el[0][1]=-1.0;
	   VR_Sol.el[0][2]=-1.0;
	   return TP_OK;
	 }
	 else {
	   SR = sqrt(b2_4ac);
	   if (!msgError)
		fprintf(stderr,"cubicRSolution - TP_ERROR - Quadratic     (X1 X2 -1)    \n");
	   VR_Sol.el[0][0]= (-c - SR)/2/b;
	   VR_Sol.el[0][1]= (-c + SR)/2/b;
	   VR_Sol.el[0][2]= -1.0;
	   return TP_OK;
	 }
    } /* FROM (b==0)  */
  }
  else {
    /* Transformation Y^3 + A Y = B */
    aM = (c-b*b/3/a)/a;
    bM = -(d + 2*b*b*b/27/a/a - b*c/3/a)/a;
    
    uAux = bM*bM + 4*aM*aM*aM/27;
    if (uAux>=0) {
	 u = (-bM + sqrt(uAux))/2;
	 t = cbrt(u);
	 if (t==0)
        s=cbrt(bM+u);
	 else
        s=aM/3/t;
	 /* Solution Y to  Y^3 + A Y = B */
	 Y= s-t;
    }
    else {
	 /* We need to find the Reel Solution */
	 /* uAux change sign for SQRT*/
	 uAux = -uAux;
	 /* u ---> uComp */
	 uComp.re   = -bM/2;
	 uComp.im   = sqrt(uAux)/2;
	 uComp.mod  = sqrt(uComp.re*uComp.re + uComp.im*uComp.im);
	 uComp.teta = atan2(uComp.im,uComp.re);
	 /* Find Cubic Root of complex number */
	 tComp.re = cbrt(uComp.mod)*cos(uComp.teta/3);
	 tComp.im = cbrt(uComp.mod)*sin(uComp.teta/3);
	 /* Find s Complex */
	 sComp.re =  aM*tComp.re/3/(tComp.re*tComp.re + tComp.im*tComp.im);
	 sComp.im = -aM*tComp.im/3/(tComp.re*tComp.re + tComp.im*tComp.im);
	 /* Find Y Complex */
	 YComp.re = sComp.re - tComp.re;
	 YComp.im = sComp.im - tComp.im;
	 if (fabs(YComp.im)<0.0001) 
	  Y = YComp.re;
	 else {
	   VR_Y3 = newdmat(0,0,0,3,&errno);
	   if (errno) {
		fprintf(stderr,"cubicRSolution - TP_ERROR - VR_Y2 Definition \n");
		VR_Sol.el[0][0]=-1.0;
		VR_Sol.el[0][1]=-1.0;
		VR_Sol.el[0][2]=-1.0;
		return TP_ERROR;
	   }
	   VR_Y2 = newdmat(0,0,0,2,&errno);
	   if (errno) {
		fprintf(stderr,"cubicRSolution - TP_ERROR - VR_Y2 Definition \n");
		VR_Sol.el[0][0]=-1.0;
		VR_Sol.el[0][1]=-1.0;
		VR_Sol.el[0][2]=-1.0;
		freemat(VR_Y3);
		return TP_ERROR;
	   }
	   VR_D = newdmat(0,0,0,1,&errno);
	   if (errno) {
		fprintf(stderr,"cubicRSolution - TP_ERROR - VR_D Definition \n");
		VR_Sol.el[0][0]=-1.0;
		VR_Sol.el[0][1]=-1.0;
		VR_Sol.el[0][2]=-1.0;
		freemat(VR_Y2);
		freemat(VR_Y3);
		return TP_ERROR;
	   }
	   VR_R = newdmat(0,0,0,1,&errno);
	   if (errno) {
		fprintf(stderr,"cubicRSolution - TP_ERROR - VR_R Definition \n");
		VR_Sol.el[0][0]=-1.0;
		VR_Sol.el[0][1]=-1.0;
		VR_Sol.el[0][2]=-1.0;
		freemat(VR_D);
		freemat(VR_Y2);
		freemat(VR_Y3);
		return TP_ERROR;
	   }
	   /* Y3 Vector */
	   VR_Y3.el[0][0]= 1.0;
	   VR_Y3.el[0][1]= 0.0;
	   VR_Y3.el[0][2]= aM;
	   VR_Y3.el[0][3]= -bM;
	   /* Y2 Vector */
	   VR_Y2.el[0][0]= 1.0;
	   VR_Y2.el[0][1]= -2*YComp.re*YComp.re;
	   VR_Y2.el[0][2]= YComp.re*YComp.re + YComp.im*YComp.im;
	   if (deConvolution(VR_Y3,VR_Y2,VR_D,VR_R)!=TP_OK) {
		fprintf(stderr, "cubicRSolution - TP_ERROR deConvolution \n");
		VR_Sol.el[0][0]=-1.0;
		VR_Sol.el[0][1]=-1.0;
		VR_Sol.el[0][2]=-1.0;
		freemat(VR_R);
		freemat(VR_D);
		freemat(VR_Y2);
		freemat(VR_Y3);
		return TP_ERROR;
	   }
	   /* Module (Reminder) calcul */
	   modR = VR_R.el[0][0]*VR_R.el[0][0] + VR_R.el[0][1]*VR_R.el[0][1];
	   /* Root verification */
	   if (modR>0.001 || VR_D.el[0][1]==0.0 ) {
		if (!msgError)
		  fprintf(stderr, "cubicRSolution - TP_ERROR Can't find solution \n");
		VR_Sol.el[0][0]=-1.0;
		VR_Sol.el[0][1]=-1.0;
		VR_Sol.el[0][2]=-1.0;
		freemat(VR_R);
		freemat(VR_D);
		freemat(VR_Y2);
		freemat(VR_Y3);
		return TP_OK;
	   }

	   Y= VR_D.el[0][0]/VR_D.el[0][1];
	   X= Y-b/3/a;
	   if (!msgError)
		fprintf(stderr,"cubicRSolution - TP_ERROR Two Imaginary Roots (X -1 -1) Sol by Conv \n");
	   VR_Sol.el[0][0]= X;
	   VR_Sol.el[0][1]=-1.0;
	   VR_Sol.el[0][2]=-1.0;
	   freemat(VR_R);
	   freemat(VR_D);
	   freemat(VR_Y2);
	   freemat(VR_Y3);
	   return TP_OK;
	 }
    } /* FROM  (uAux>=0) */
    /* Solution X to  a X^3 + b X^2 + c X + d = 0 */
    X= Y-b/3/a;
    /* Quadratic Pol Definition */
    VR_Quad = newdmat(0,0,0,2,&errno);
    if (errno) {
	 if (!msgError)
	   fprintf(stderr,"cubicRSolution - TP_ERROR - VR_Quad Definition \n");
	 VR_Sol.el[0][0]=-1.0;
	 VR_Sol.el[0][1]=-1.0;
	 VR_Sol.el[0][2]=-1.0;
	 return TP_ERROR;
    }
    /* Find Quadratic Pol Coefficients */
    if (SynteticDivision(VR_Pol,X, VR_Quad, &reminder)!=TP_OK) {
	 if (!msgError)
	   fprintf(stderr,"cubicRSolution - TP_ERROR - SynteticDivision \n");
	 VR_Sol.el[0][0]=-1.0;
	 VR_Sol.el[0][1]=-1.0;
	 VR_Sol.el[0][2]=-1.0;
	 freemat(VR_Quad);
	 return TP_ERROR;
    }
    /* X is a ROOT, verification */
    if (fabs(reminder)>0.001) {
	 if (!msgError)
	   fprintf(stderr,"cubicRSolution - TP_ERROR Can't find Solution\n");
	 freemat(VR_Quad);
	 VR_Sol.el[0][0]=-1.0;
	 VR_Sol.el[0][1]=-1.0;
	 VR_Sol.el[0][2]=-1.0;
	 return TP_OK;
    }
    else {
	 /* Solving Quadratic */
	 a = VR_Quad.el[0][0];
	 b = VR_Quad.el[0][1];
	 c = VR_Quad.el[0][2];
	 b2_4ac=b*b-4*a*c;
	 if (b2_4ac <0) {
	   /* Complex Solution, X is the real solution */
	   if (!msgError)
		fprintf(stderr,"cubicRSolution - TP_ERROR Two Imaginary Roots (X -1 -1) \n");
	   VR_Sol.el[0][0]= X;
	   VR_Sol.el[0][1]=-1.0;
	   VR_Sol.el[0][2]=-1.0;
	   freemat(VR_Quad);
	   return TP_OK;
	 }
	 else {
	   SR = sqrt(b2_4ac);
	   /* Three real solutions */
	   VR_Sol.el[0][0]= X;
	   VR_Sol.el[0][1]= (-b + SR)/2/a;
	   VR_Sol.el[0][2]= (-b - SR)/2/a;
	 }
    } /* FROM (fabs(reminder)>0.001) */
    /* Memory Liberation */
    freemat(VR_Quad);
  } /* FROM   (a==0)   */
  return TP_OK;
}

/* QuarticRSolution */
/* This function returns the REAL roots of                                     */
/*                  a X^4 + b X^3 + c X^2 + d X + e = 0                        */
/* Note:                                                                       */
/*      IT'S NO A GENERAL SOLUTION                                             */
/*      OUR APPLICATION ONLY DEMANDS POSITIVE ROOTS                            */
/*      THEN COMPLEX ROOTS EQUALS TO -1                                        */
/*      CONFUSION PROBLEM WHEN REAL ROOTS EQUALS TO -1                         */
/*      RETURNS TP_ERROR                                                          */
/*      POSSIBLE SOLUTION, EVIDENT                                             */
/*      WHEN POL IS QUADRATIC RETURNS -1 FOR THE FIRST ROOT AND TP_ERROR          */
/*               IS MONOMIAL  RETURNS THE ROOT AND -1 FOR THE OTHERS AND TP_ERROR */
/*               IS CONSTANTS RETURNS -1 FOR THE ROOTS AND TP_ERROR               */
/*                                                                             */
/* Input :     dmat VR_POL     Coefficients Poly                               */
/* Output:     dmat VR_SOL     Roots Vector                                    */
                                
TP_STATUS quarticRSolution(dmat VR_Pol, dmat VR_Sol, TP_SELECT msgError )
{
  double a, b, c, d, e;
  double a3, a2, a1, a0;
  double cq2, cq1, cq0;
  double Y, auxR, auxD, auxE;
  TP_COMPLEX RComp, DComp, EComp, S1Comp, S2Comp, S3Comp, S4Comp, AuxComp; 
  dmat VR_CubicPol, VR_CubicSol;
  int errno;
  int flagImag=0;

  /* Verifie dimensions vector */
  if ((VR_Pol.ub2-VR_Pol.lb2) != 4 || (VR_Pol.ub1-VR_Pol.lb1) != 0) {
    fprintf(stderr,"quarticRSolution - TP_ERROR - Poly Coefficients Error\n");
    return TP_ERROR;
  }

  if ((VR_Sol.ub2-VR_Sol.lb2) != 3 || (VR_Sol.ub1-VR_Sol.lb1) != 0) {
    fprintf(stderr,"quarticRSolution - TP_ERROR - Dimension Solutions Error\n");
    return TP_ERROR;
  }

  /* Coefficients assignation */
  /*   a X^4 + b X^3 + c X^2 + d X + e = 0   */
  a = VR_Pol.el[0][0];
  b = VR_Pol.el[0][1];
  c = VR_Pol.el[0][2];
  d = VR_Pol.el[0][3];
  e = VR_Pol.el[0][4]; 

  if (a==0) {
    /* Cubic poly */
    VR_CubicPol = newdmat(0,0,0,3,&errno);
    if (errno) {
	   fprintf(stderr,"quarticRSolution - TP_ERROR - VR_CubicPol Definition \n");
	   VR_Sol.el[0][0]=-1.0;
	   VR_Sol.el[0][1]=-1.0;
	   VR_Sol.el[0][2]=-1.0;
	   VR_Sol.el[0][3]=-1.0;
	   return TP_ERROR;
    }
    VR_CubicSol = newdmat(0,0,0,2,&errno);
    if (errno) {
	   fprintf(stderr,"quarticRSolution - TP_ERROR - VR_CubicSol Definition \n");
	   VR_Sol.el[0][0]=-1.0;
	   VR_Sol.el[0][1]=-1.0;
	   VR_Sol.el[0][2]=-1.0;
	   VR_Sol.el[0][3]=-1.0;
	   freemat(VR_CubicPol);
	   return TP_ERROR;
    }
    VR_CubicPol.el[0][0] = b;
    VR_CubicPol.el[0][1] = c;
    VR_CubicPol.el[0][2] = d;
    VR_CubicPol.el[0][3] = e;

    if (cubicRSolution(VR_CubicPol, VR_CubicSol, msgError)!=TP_OK) {
	 if (!msgError)
	   fprintf(stderr,"quarticRSolution TP_ERROR - cubicRSolution \n");
	 VR_Sol.el[0][0]= -1.0;
	 VR_Sol.el[0][1]= -1.0;
	 VR_Sol.el[0][2]= -1.0;
	 VR_Sol.el[0][3]= -1.0;
	 freemat(VR_CubicSol);
	 freemat(VR_CubicPol);
	 return TP_ERROR;
    }
    VR_Sol.el[0][0]= VR_CubicSol.el[0][0];
    VR_Sol.el[0][1]= VR_CubicSol.el[0][1];
    VR_Sol.el[0][2]= VR_CubicSol.el[0][2];
    VR_Sol.el[0][3]= -1.0;
    if (!msgError)
	 fprintf(stderr,"quartic Solution - TP_ERROR - solution [X1 X2 X3 -1] \n");
    freemat(VR_CubicSol);
    freemat(VR_CubicPol);
    return TP_OK;
  } /* FROM a = 0 */
  else {
    /* Normalized Poly */
    /* Z^4 + a3*Z^3 + a2*z^2 + a1*z + a0 = 0 */
    a3 = b/a;
    a2 = c/a;
    a1 = d/a;
    a0 = e/a;

    /* Auxiliar Poly */
    /* Y^3 -a2*Y^2 + (a1*a3 - 4*a0)*Y + (4*a2*a0 - a1^2 -a3^2*a0) = 0 */
    cq2 = - a2;
    cq1 = a1*a3 - 4*a0;
    cq0 = 4*a2*a0 - a1*a1 -a3*a3*a0;
    /* Cubic poly */
    VR_CubicPol = newdmat(0,0,0,3,&errno);
    if (errno) {
	   fprintf(stderr,"quarticRSolution - TP_ERROR - VR_CubicPol Definition 2 \n");
	   VR_Sol.el[0][0]=-1.0;
	   VR_Sol.el[0][1]=-1.0;
	   VR_Sol.el[0][2]=-1.0;
	   VR_Sol.el[0][3]=-1.0;
	   return TP_ERROR;
    }
    VR_CubicSol = newdmat(0,0,0,2,&errno);
    if (errno) {
	   fprintf(stderr,"quarticRSolution - TP_ERROR - VR_CubicSol Definition 2 \n");
	   VR_Sol.el[0][0]=-1.0;
	   VR_Sol.el[0][1]=-1.0;
	   VR_Sol.el[0][2]=-1.0;
	   VR_Sol.el[0][3]=-1.0;
	   freemat(VR_CubicPol);
	   return TP_ERROR;
    }
    VR_CubicPol.el[0][0] = 1;
    VR_CubicPol.el[0][1] = cq2;
    VR_CubicPol.el[0][2] = cq1;
    VR_CubicPol.el[0][3] = cq0;

    if (cubicRSolution(VR_CubicPol, VR_CubicSol, msgError)!=TP_OK) {
	 if (!msgError)
	   fprintf(stderr,"quarticRSolution TP_ERROR - cubicRSolution 2\n");
	 VR_Sol.el[0][0]= -1.0;
	 VR_Sol.el[0][1]= -1.0;
	 VR_Sol.el[0][2]= -1.0;
	 VR_Sol.el[0][3]= -1.0;
	 freemat(VR_CubicSol);
	 freemat(VR_CubicPol);
	 return TP_ERROR;
    }
    /* Solving for Y */
    Y = VR_CubicSol.el[0][0];

    freemat(VR_CubicSol);
    freemat(VR_CubicPol);

    auxR = a3*a3/4-a2+Y;
    if (auxR==0) {
	 auxD = Y*Y - 4*a0;
	 RComp.re = 0.0;
	 RComp.im = 0.0;
	 if (auxD<0) {
	   AuxComp.re = 3*a3*a3/4 - 2*a2;
	   AuxComp.im = 2*sqrt(-auxD);
	   AuxComp.mod = sqrt(AuxComp.re*AuxComp.re + AuxComp.im*AuxComp.im);
	   AuxComp.teta = atan2(AuxComp.im,AuxComp.re);
	   DComp.re = sqrt(AuxComp.mod)*cos(AuxComp.teta/2);
	   DComp.im = sqrt(AuxComp.mod)*sin(AuxComp.teta/2);
	   EComp.re = sqrt(AuxComp.mod)*cos(AuxComp.teta/2);
	   EComp.im =-sqrt(AuxComp.mod)*sin(AuxComp.teta/2);
	 } /* FROM auxD<0 */
	 else {
	   auxE = 3*a3*a3/4 - 2*a2 + 2*sqrt(auxD);
	   if (auxE>=0) {
		DComp.re = sqrt(auxE);
		DComp.im = 0;
	   }
	   else {
		DComp.re = 0;
		DComp.im = sqrt(-auxE);
	   }
	   auxE = 3*a3*a3/4 - 2*a2 - 2*sqrt(auxD);
	   if (auxE>=0) {
		EComp.re = sqrt(auxE);
		EComp.im = 0;
	   }
	   else {
		EComp.re = 0;
		EComp.im = sqrt(-auxE);
	   }
	 }/* FROM auxD<0 */
    } /* FROM auxR=0 */
    else {
	 if (auxR>0) {
	   RComp.re = sqrt(auxR);
	   RComp.im = 0;
	   auxD = 0.75*a3*a3 - RComp.re*RComp.re - 2*a2 + 0.25*(4*a3*a2 - 8*a1 -a3*a3*a3)/RComp.re;
	   if (auxD>=0) {
		DComp.re = sqrt(auxD);
		DComp.im = 0;
	   }
	   else {
		DComp.re = 0;
		DComp.im = sqrt(-auxD);
	   }
	   auxE = 0.75*a3*a3 - RComp.re*RComp.re - 2*a2 - 0.25*(4*a3*a2 - 8*a1 -a3*a3*a3)/RComp.re;
	   if (auxE>=0) {
		EComp.re = sqrt(auxE);
		EComp.im = 0;
	   }
	   else {
		EComp.re = 0;
		EComp.im = sqrt(-auxE);
	   }
	 } /* FROM auxR>0 */
	 else {
	   RComp.re = 0;
	   RComp.im = sqrt(-auxR);
	   AuxComp.re = 0.75*a3*a3 + RComp.im*RComp.im - 2*a2 ;
	   AuxComp.im = -0.25*(4*a3*a2 - 8*a1 -a3*a3*a3)/RComp.im;
	   AuxComp.mod = sqrt(AuxComp.re*AuxComp.re + AuxComp.im*AuxComp.im);
	   AuxComp.teta = atan2(AuxComp.im,AuxComp.re);
	   DComp.re = sqrt(AuxComp.mod)*cos(AuxComp.teta/2);
	   DComp.im = sqrt(AuxComp.mod)*sin(AuxComp.teta/2);
	   EComp.re = sqrt(AuxComp.mod)*cos(AuxComp.teta/2);
	   EComp.im =-sqrt(AuxComp.mod)*sin(AuxComp.teta/2);
	 } /* FROM auxR>0 */
    } /* FROM auxR=0 */
  } /* FROM a = 0 */
  /* Complex Solutions */
  S1Comp.re = -a3/4 + RComp.re/2 + DComp.re/2;
  S1Comp.im = RComp.im/2 + DComp.im/2;
  S2Comp.re = -a3/4 + RComp.re/2 - DComp.re/2;
  S2Comp.im = RComp.im/2 - DComp.im/2;
  S3Comp.re = -a3/4 - RComp.re/2 + EComp.re/2;
  S3Comp.im = -RComp.im/2 + EComp.im/2;
  S4Comp.re = -a3/4 - RComp.re/2 - EComp.re/2;
  S4Comp.im = -RComp.im/2 - EComp.im/2;

  /* Roots to Vector Solution */
  if (fabs(S1Comp.im)<0.001) VR_Sol.el[0][0] = S1Comp.re;
  else {
    VR_Sol.el[0][0] = -1.0;
    flagImag = 1;
  }
  if (fabs(S2Comp.im)<0.001) VR_Sol.el[0][1] = S2Comp.re;
  else {
    VR_Sol.el[0][1] = -1.0;
    flagImag = 1;
  }
  if (fabs(S3Comp.im)<0.001) VR_Sol.el[0][2] = S3Comp.re;
  else {
    VR_Sol.el[0][2] = -1.0;
    flagImag = 1;
  }
  if (fabs(S4Comp.im)<0.001) VR_Sol.el[0][3] = S4Comp.re;
  else {
    VR_Sol.el[0][3] = -1.0;
    flagImag = 1;
  }
  if (flagImag==1 &&  !msgError ) fprintf(stderr,"quarticRSolution - TP_ERROR - Imaginary ROOTS -1 \n");
  return TP_OK;
}

/* Quaternion Time Rates to Angular Velocity
              quat2omega(TP_QUAT q, TP_QUAT dq, TP_VEL *omega)
   
		    omega = 2 Mat_QR(q)' * dq

   Introduction to Intelligent Robotics
   Herman Bruyninckx and Joris De Schutter
*/
TP_STATUS quat2omega(TP_QUAT q, TP_QUAT dq, TP_VEL *omega)
{
  double s, x, y, z;
  double vs, vx, vy, vz;
  double wx, wy, wz;

  s = q.n;
  x = q.i;
  y = q.j;
  z = q.k;

  vs = dq.n;
  vx = dq.i;
  vy = dq.j;
  vz = dq.k;

  wx = 2*(  s*vx - z*vy + y*vz - x*vs );
  wy = 2*(  z*vx + s*vy - x*vz - y*vs );
  wz = 2*(- y*vx + x*vy + s*vz - z*vs );

  omega->wx = wx;
  omega->wy = wy;
  omega->wz = wz;

  return TP_OK;
}
/* 
   Rotation Matrix 2 Quaternion

   Based on gbJido - Daniel SIDOBRE LAAS
*/
TP_STATUS RotMat2Quat(Gb_th* Mat, TP_QUAT *Quat)
{
  double s0, s1, s2, s3, norme;
  s0 = Mat->vx.x + Mat->vy.y + Mat->vz.z;
  if (s0 > -0.1) {
    Quat->n = sqrt(1 + s0) / 2.;
    Quat->i = (Mat->vy.z - Mat->vz.y) / (4 * Quat->n);
    Quat->j = (Mat->vz.x - Mat->vx.z) / (4 * Quat->n);
    Quat->k = (Mat->vx.y - Mat->vy.x) / (4 * Quat->n);
  } else {
    s1 = Mat->vx.x - Mat->vy.y - Mat->vz.z;
    if (s1 > -0.1) {
      Quat->i = sqrt(1 + s1) / 2.;
      Quat->n = (Mat->vy.z - Mat->vz.y) / (4 * Quat->i);
      Quat->j = (Mat->vx.y + Mat->vy.x) / (4 * Quat->i);
      Quat->k = (Mat->vx.z + Mat->vz.x) / (4 * Quat->i);
    } else {
      s2 = - Mat->vx.x + Mat->vy.y - Mat->vz.z;
      if (s2 > -0.1) {
	   Quat->j = sqrt(1 + s2) / 2.;
	   Quat->n = (Mat->vz.x - Mat->vx.z) / (4 * Quat->j);
	   Quat->i = (Mat->vx.y + Mat->vy.x) / (4 * Quat->j);
	   Quat->k = (Mat->vz.y + Mat->vy.z) / (4 * Quat->j);
      } else {
	   s3 = - Mat->vx.x - Mat->vy.y + Mat->vz.z;
	   if (s3 < -0.1) {
		fprintf(stderr, "RotMat 2 Quat - TP_ERROR - s3 < -0.1 !!!\n");
		return TP_ERROR;
	   }
	   Quat->k = sqrt(1 + s3) / 2.;
	   Quat->n = (Mat->vx.y - Mat->vy.x) / (4 * Quat->k);
	   Quat->i = (Mat->vx.z + Mat->vz.x) / (4 * Quat->k);
	   Quat->j = (Mat->vz.y + Mat->vy.z) / (4 * Quat->k);
      }
    }
  }

  norme = sqrt( Quat->n*Quat->n + Quat->i*Quat->i + Quat->j*Quat->j + Quat->k*Quat->k);

  Quat->n = Quat->n/norme;
  Quat->i = Quat->i/norme;
  Quat->j = Quat->j/norme;
  Quat->k = Quat->k/norme;

  return TP_OK;
}


/* Singular Robust Inverse for squared Matrix */
/* Nakamura */
/* Inv = A'* inv(A*A' + lambda*I) */
TP_STATUS SRSQ_Inv(dmat A, dmat InvA, dmat Aux, double lambda) {
  register int i, j, k, rowsize;
  
  rowsize= A.ub2-A.lb2;

  /* Aux = A * A' + lambda I */
  for (i = 0; i <= rowsize; i++) {
    for (j = 0; j <= rowsize; j++) {
      Aux.el[Aux.lb1+i][Aux.lb2+j] = 0;
      for (k = 0; k <= rowsize; k++) {
	Aux.el[Aux.lb1+i][Aux.lb2+j] = Aux.el[Aux.lb1+i][Aux.lb2+j] + A.el[A.lb1+i][A.lb2+k]* A.el[A.lb1+j][A.lb2+k];
      }
      if (i==j) {
	Aux.el[Aux.lb1+i][Aux.lb2+j] = Aux.el[Aux.lb1+i][Aux.lb2+j] + lambda;
      }
    }
  }
  
  /* Aux = inverse (Aux) */
  if (matinvert(Aux)==0.0) {
    fprintf(stderr,"SR Squared Inverse - TP_ERROR - Det Null of Inv Matrix \n");
    return TP_ERROR;
  }
  
  /* InvA =  A' * Aux */
  for (i = 0; i <= rowsize; i++) {
    for (j = 0; j <= rowsize; j++) {
      InvA.el[InvA.lb1+i][InvA.lb2+j] = 0;
      for (k = 0; k <= rowsize; k++) {
	InvA.el[InvA.lb1+i][InvA.lb2+j] = InvA.el[InvA.lb1+i][InvA.lb2+j] + A.el[A.lb1+k][A.lb2+i]* Aux.el[Aux.lb1+k][Aux.lb2+j];
      }
    }
  }

  return TP_OK;
}

TP_STATUS quaternionNormalize(TP_QUAT* quat) {
  double norm;
  norm = quat->n * quat->n  + quat->i * quat->i + quat->j * quat->j  + quat->k * quat->k ;
  norm=sqrt(norm);
  quat->n = quat->n/norm;
  quat->i = quat->i/norm;
  quat->j = quat->j/norm;
  quat->k = quat->k/norm;
  return TP_OK;  
}

TP_STATUS calculOfQuaternionDerivative(double WxRef, double WyRef, double WzRef, const TP_QUAT* q, TP_QUAT* dq) {

  //    double Omega[4][4];
  //    Omega[0][0] = 0;
  //    Omega[0][1] = -WxRef;
  //    Omega[0][2] = -WyRef;
  //    Omega[0][3] = -WzRef;
  //    Omega[1][0] = WxRef;
  //    Omega[1][1] = 0;
  //    Omega[1][2] = WzRef;
  //    Omega[1][3] = -WyRef;
  //    Omega[2][0] = WyRef;
  //    Omega[2][1] = -WzRef;
  //    Omega[2][2] = 0;
  //    Omega[2][3] = WxRef;
  //    Omega[3][0] = WzRef;
  //    Omega[3][1] = WyRef;
  //    Omega[3][2] = -WxRef;
  //    Omega[3][3] = 0;

  dq->n = 0.5 * ( 0             + (-WxRef * q->i) + (-WyRef * q->j) + (-WzRef * q->k));
  dq->i = 0.5 * ((WxRef * q->n) +   0             + ( WzRef * q->j) + (-WyRef * q->k));
  dq->j = 0.5 * ((WyRef * q->n) + (-WzRef * q->i) +   0             + ( WxRef * q->k));
  dq->k = 0.5 * ((WzRef * q->n) + ( WyRef * q->i) + (-WxRef * q->j) +   0            );

  return TP_OK;
}
